﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data.SqlClient;
using System.Data.Sql;
using System.Data.OleDb;

namespace CcwTechnologies.Windows.Forms.SqlServer
{
    public partial class ConnectToServerForm : Form
    {
        public bool IsConnectionValid { get; private set; }
        public string OleConnectionString { get; private set; }
        public string AdoConnectionString { get; private set; }
        public string DatabaseName { get; private set; }

        public ConnectToServerForm()
        {
            InitializeComponent();
        }

        #region Methods [private]

        /// <summary>
        /// Gets a connection string using the information currently provided on the form.
        /// </summary>
        /// <returns></returns>
        private string GetConnectionString(bool includeDatabase)
        {
            string connectionString;
            if (this.windowsAuthentication.Checked)
                connectionString = string.Format("Server={0}; Integrated Security=SSPI;", sqlServerComboBox.Text);
            else
                connectionString = string.Format("Server={0}; User ID={1}; Password={2};", sqlServerComboBox.Text, usernameTextBox.Text, passwordTextBox.Text);

            if ( includeDatabase && !string.IsNullOrEmpty(this.databaseComboBox.Text))
                connectionString += string.Format(" Database={0};", this.databaseComboBox.Text);

            return connectionString;
        }

        private string GetOleConnectionString()
        {
            return string.Format("{0} Provider=SQLOLEDB;", this.GetConnectionString(true));
        }



        #endregion

        #region Event Handlers

        /// <summary>
        /// Occurs when the 'testConnectionButton' button is clicked.
        /// </summary>
        private void testConnectionButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(this.sqlServerComboBox.Text))
            {
                MessageBox.Show("An SQL server must be specified.");
                return;
            }

            if (sqlServerAuthentication.Checked && string.IsNullOrEmpty(this.usernameTextBox.Text))
            {
                MessageBox.Show("If SQL server authentication is used, a username must be provided.");
                return;
            }

            try
            {
                using (SqlConnection sqlConnection = new SqlConnection(this.GetConnectionString(true)))
                {
                    sqlConnection.Open();
                    sqlConnection.Close();
                    MessageBox.Show("Connection successful!");
                }
            }
            catch (SqlException ex)
            {
                MessageBox.Show("Connection failed: " + ex.Message);
            }
        }

        /// <summary>
        /// Occurs when the 'cancelButton' button is clicked.
        /// </summary>
        private void cancelButton_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        /// <summary>
        /// Occurs when the authentication type radio buttons 'sqlServerAuthentication' and 'windowsAuthentication' are changed.
        /// </summary>
        private void sqlServerAuthentication_CheckedChanged(object sender, EventArgs e)
        {
            usernameTextBox.Enabled =
                passwordTextBox.Enabled =
                sqlServerAuthentication.Checked;
        }

        /// <summary>
        /// Occurs when the 'databaseComboBox' dropdown is expanded.
        /// </summary>
        private void databaseComboBox_DropDown(object sender, EventArgs e)
        {
            try
            {
                using (SqlConnection sqlConnection = new SqlConnection(this.GetConnectionString(false)))
                {
                    sqlConnection.Open();
                    DataTable databaseList = sqlConnection.GetSchema("Databases");
                    sqlConnection.Close();

                    this.databaseComboBox.Items.Clear();
                    foreach (DataRow row in databaseList.Rows)
                        this.databaseComboBox.Items.Add(row["database_name"]);
                }
            }
            catch (SqlException)
            {
                this.databaseComboBox.Items.Clear();
            }
        }

        /// <summary>
        /// Occurs when the 'sqlServerComboBox' dropdown is expanded.
        /// </summary>
        private void sqlServerComboBox_DropDown(object sender, EventArgs e)
        {
            if (this.sqlServerComboBox.Items.Count > 0)
                return;

            DataTable instances = SqlDataSourceEnumerator.Instance.GetDataSources();
            foreach (DataRow row in instances.Rows)
            {
                string name = row["ServerName"].ToString();
                if (row["InstanceName"] != null && row["InstanceName"].ToString() != string.Empty)
                    name += string.Format(@"\{0}", row["InstanceName"]);
                this.sqlServerComboBox.Items.Add(name);
            }
        }

        private void connectButton_Click(object sender, EventArgs e)
        {
            this.IsConnectionValid = false;
            this.AdoConnectionString = null;
            this.AdoConnectionString = null;

            if (string.IsNullOrEmpty(this.sqlServerComboBox.Text))
            {
                MessageBox.Show("An SQL server must be specified.");
                return;
            }

            if (sqlServerAuthentication.Checked && string.IsNullOrEmpty(this.usernameTextBox.Text))
            {
                MessageBox.Show("If SQL server authentication is used, a username must be provided.");
                return;
            }

            if (string.IsNullOrEmpty(databaseComboBox.Text))
            {
                MessageBox.Show("A database name must be provided.");
                return;
            }

            try
            {
                using (OleDbConnection connection = new OleDbConnection(this.GetOleConnectionString()))
                {
                    connection.Open();
                    connection.Close();
                    this.AdoConnectionString = this.GetConnectionString(true);
                    this.OleConnectionString = this.GetOleConnectionString();
                    this.IsConnectionValid = true;
                    this.DatabaseName = this.databaseComboBox.Text;
                }
            }
            catch (OleDbException ex)
            {
                MessageBox.Show("Connection failed: " + ex.Message);
            }

            this.Close();
        }

        #endregion
    }
}
